Enum object_store::azure::AzureConfigKey
source · #[non_exhaustive]pub enum AzureConfigKey {
Show 16 variants
AccountName,
AccessKey,
ClientId,
ClientSecret,
AuthorityId,
SasKey,
Token,
UseEmulator,
UseFabricEndpoint,
MsiEndpoint,
ObjectId,
MsiResourceId,
FederatedTokenFile,
UseAzureCli,
ContainerName,
Client(ClientConfigKey),
}Expand description
Configuration keys for MicrosoftAzureBuilder
Configuration via keys can be done via MicrosoftAzureBuilder::with_config
Example
let builder = MicrosoftAzureBuilder::new()
.with_config("azure_client_id".parse().unwrap(), "my-client-id")
.with_config(AzureConfigKey::AuthorityId, "my-tenant-id");Variants (Non-exhaustive)
This enum is marked as non-exhaustive
AccountName
The name of the azure storage account
Supported keys:
azure_storage_account_nameaccount_name
AccessKey
Master key for accessing storage account
Supported keys:
azure_storage_account_keyazure_storage_access_keyazure_storage_master_keyaccess_keyaccount_keymaster_key
ClientId
Service principal client id for authorizing requests
Supported keys:
azure_storage_client_idazure_client_idclient_id
ClientSecret
Service principal client secret for authorizing requests
Supported keys:
azure_storage_client_secretazure_client_secretclient_secret
AuthorityId
Tenant id used in oauth flows
Supported keys:
azure_storage_tenant_idazure_storage_authority_idazure_tenant_idazure_authority_idtenant_idauthority_id
SasKey
Shared access signature.
The signature is expected to be percent-encoded, much like they are provided in the azure storage explorer or azure portal.
Supported keys:
azure_storage_sas_keyazure_storage_sas_tokensas_keysas_token
Token
Bearer token
Supported keys:
azure_storage_tokenbearer_tokentoken
UseEmulator
Use object store with azurite storage emulator
Supported keys:
azure_storage_use_emulatorobject_store_use_emulatoruse_emulator
UseFabricEndpoint
Use object store with url scheme account.dfs.fabric.microsoft.com
Supported keys:
azure_use_fabric_endpointuse_fabric_endpoint
MsiEndpoint
Endpoint to request a imds managed identity token
Supported keys:
azure_msi_endpointazure_identity_endpointidentity_endpointmsi_endpoint
ObjectId
Object id for use with managed identity authentication
Supported keys:
azure_object_idobject_id
MsiResourceId
Msi resource id for use with managed identity authentication
Supported keys:
azure_msi_resource_idmsi_resource_id
FederatedTokenFile
File containing token for Azure AD workload identity federation
Supported keys:
azure_federated_token_filefederated_token_file
UseAzureCli
Use azure cli for acquiring access token
Supported keys:
azure_use_azure_cliuse_azure_cli
ContainerName
Container name
Supported keys:
azure_container_namecontainer_name
Client(ClientConfigKey)
Client options