Module prefix
object_
store
0.14.1
Module prefix
Module Items
Structs
In crate object_
store
object_store
Module
prefix
An object store wrapper handling a constant path prefix
Structs

Prefix
Store
Store wrapper that applies a constant prefix to all paths handled by the store.